package com.care.bms.mqtt;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.care.bms.service.BmsEventOrderService;
import com.care.common.entity.CareDevice;
import com.care.common.enums.DeviceActiveStatusEnum;
import com.care.common.service.*;
import com.care.common.util.CommonConfUtil;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MqttTool {
    private static Map<String, MqttDataConnector> mqttDataConnectorMap = new HashMap<>();
    public static void init(CareDeviceService careDeviceService,
                            BmsEventOrderService bmsEventOrderService,
                            CareEventOrderService careEventOrderService,
                            CareHouseService careHouseService,
                            CareOrganizationService careOrganizationService,
                            CareStationService careStationService,
                            CareMqttMsgService careMqttMsgService) {
        QueryWrapper<CareDevice> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().eq(CareDevice::getActiveStatus, DeviceActiveStatusEnum.ACTIVE.getValue());
        List<CareDevice> listCareDevice = careDeviceService.list(queryWrapper);
        for (CareDevice careDevice : listCareDevice) {
            addOne(careDevice,careDeviceService,bmsEventOrderService,careEventOrderService,careHouseService,careOrganizationService,careStationService,careMqttMsgService);
        }
    }

    /**
     * 重新初始化
     * @param careDeviceService
     * @param bmsEventOrderService
     * @param careEventOrderService
     * @param careHouseService
     * @param careOrganizationService
     * @param careStationService
     */
    public static void reInit(CareDeviceService careDeviceService,
                              BmsEventOrderService bmsEventOrderService,
                              CareEventOrderService careEventOrderService,
                              CareHouseService careHouseService,
                              CareOrganizationService careOrganizationService,
                              CareStationService careStationService,
                              CareMqttMsgService careMqttMsgService) {
        mqttDataConnectorMap.forEach((key, value) -> {
            value.destroy();
            mqttDataConnectorMap.remove(key);
        });
        init(careDeviceService,bmsEventOrderService,careEventOrderService,careHouseService,careOrganizationService,careStationService,careMqttMsgService);
    }

    /**
     * 添加一个设备到服务
     * @param careDeviceService
     * @param bmsEventOrderService
     * @param careEventOrderService
     * @param careHouseService
     * @param careOrganizationService
     * @param careStationService
     */
    public static void addOne(CareDevice careDevice,
                              CareDeviceService careDeviceService,
                              BmsEventOrderService bmsEventOrderService,
                              CareEventOrderService careEventOrderService,
                              CareHouseService careHouseService,
                              CareOrganizationService careOrganizationService,
                              CareStationService careStationService,
                              CareMqttMsgService careMqttMsgService) {
        if (StringUtils.isEmpty(careDevice.getTopic()) || StringUtils.isEmpty(careDevice.getClientId())) {
            return;
        }
        MqttDataConnector xbMqttDataConnector = new MqttDataConnector();
        Map<String, String> configParams = new HashMap<>();
        String host = "ws://" + CommonConfUtil.getConf("mqtt.broker") + ":" + CommonConfUtil.getConf("mqtt.port");
        String cliendId = careDevice.getClientId();
        String username = CommonConfUtil.getConf("mqtt.username");
        String password = CommonConfUtil.getConf("mqtt.password");
        String topic = careDevice.getTopic();
        configParams.put("MQTT_HOST", host);
        configParams.put("MQTT_CLIENTID", cliendId);
        configParams.put("MQTT_USERNAME", username);
        configParams.put("MQTT_PASSWORD", password);
        try {
            xbMqttDataConnector.createIotDataSource(configParams);
            MqttMessageListener mqttMessageListener = new MqttMessageListener();
            mqttMessageListener.setCareDeviceService(careDeviceService);
            mqttMessageListener.setBmsEventOrderService(bmsEventOrderService);
            mqttMessageListener.setCareEventOrderService(careEventOrderService);
            mqttMessageListener.setCareHouseService(careHouseService);
            mqttMessageListener.setCareOrganizationService(careOrganizationService);
            mqttMessageListener.setCareStationService(careStationService);
            mqttMessageListener.setCareMqttMsgService(careMqttMsgService);
            mqttMessageListener.setThreadMap(new HashMap<>());

            xbMqttDataConnector.setReSubscribe(topic, mqttMessageListener);
            mqttDataConnectorMap.put(cliendId, xbMqttDataConnector);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 从服务中去除一个设备
     */
    public static void removeOne(CareDevice careDevice) {
        String clientId = careDevice.getClientId();
        if (StringUtils.isEmpty(clientId)) {
            return;
        }
        if (mqttDataConnectorMap.get(clientId) != null) {
            mqttDataConnectorMap.get(clientId).destroy();
            mqttDataConnectorMap.remove(clientId);
        }
    }
}
