package com.care.bms.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.care.common.entity.*;

import com.care.common.service.*;
import com.care.common.util.MyBeanUtils;
import com.care.common.vo.PageReqVO;


import com.care.common.vo.memberinfo.MemberInfoVO;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 频安宝用户表(CareMemberInfo)表服务实现类
 *
 * @author makejava
 * @since 2021-05-21 00:08:38
 */
@Service
public class BmsMemberInfoService {

    @Resource
    private CareMemberInfoService careMemberInfoService;
    @Resource
    private CareDeviceService careDeviceService;

    
    /**
     * 查询频安宝用户列表
     * @return
             */
    public IPage<MemberInfoVO> listCareMemberInfo(String name, String phone, PageReqVO pageReqVo) {
            IPage<CareMemberInfo> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());
            QueryWrapper<CareMemberInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().like(StrUtil.isNotEmpty(name),CareMemberInfo::getName,name)
                    .like(StrUtil.isNotEmpty(phone),CareMemberInfo::getPhone,phone)
                    .orderByAsc(CareMemberInfo::getName);

            IPage<CareMemberInfo> pageRes = this.careMemberInfoService.page(page, queryWrapper);
            IPage<MemberInfoVO> results = new Page<>(pageRes.getCurrent(),pageRes.getSize(),pageRes.getTotal());
            if(CollUtil.isNotEmpty(pageRes.getRecords())){
                List<MemberInfoVO> list = new ArrayList<>();
                pageRes.getRecords().forEach(item -> {
                    MemberInfoVO  resVO = new MemberInfoVO();
                    BeanUtils.copyProperties(item,resVO);
                    //服务设备数量
                    QueryWrapper<CareDevice> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.lambda().eq(CareDevice::getMemberId,resVO.getId());
                    resVO.setDevCount(this.careDeviceService.count(queryWrapper1));
                    list.add(resVO);
                });
                results.setRecords(list);
            }
            return results;
    }

    /**
     * 删除频安宝用户
     * @param id
     * @return
     */
    public boolean deleteById(Long id){
        return this.careMemberInfoService.removeById(id);
    }

    /**
     * 频安宝用户详情
     * @param id
     * @return
     */
    public MemberInfoVO getCareMemberInfoById(Long id){
        CareMemberInfo careMemberInfo = this.careMemberInfoService.getById(id);
        if (careMemberInfo!=null){
            MemberInfoVO resVO = new MemberInfoVO();
            BeanUtil.copyProperties(careMemberInfo,resVO);

            //服务设备数量
            QueryWrapper<CareDevice> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.lambda().eq(CareDevice::getMemberId,resVO.getId());
            resVO.setDevCount(this.careDeviceService.count(queryWrapper1));
            return resVO;
        }
        return null;
    }


    /**
     * 保存频安宝用户
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public void createCareMemberInfo(MemberInfoVO vo){
        //保存频安宝用户
        CareMemberInfo careMemberInfo = new CareMemberInfo();
        BeanUtil.copyProperties(vo,careMemberInfo);
        careMemberInfo.setCreateTime(new Date());

        this.careMemberInfoService.save(careMemberInfo);

//        //更新设备
//        List<DeviceVO> hasDeviceVOList = vo.getHasDeviceVOList();
//        if (CollUtil.isNotEmpty(hasDeviceVOList)) {
//            hasDeviceVOList.forEach(item -> {
//                CareDevice careDevice = new CareDevice();
//                BeanUtil.copyProperties(item,careDevice);
//                careDevice.setMemberId(careMemberInfo.getId());
//                careDeviceService.updateById(careDevice);
//            });
//        }
    }

    /**
     * 修改频安宝用户
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public void updateCareMemberInfo(MemberInfoVO vo){
        //保存频安宝用户
        CareMemberInfo careMemberInfo = this.careMemberInfoService.getById(vo.getId());
        MyBeanUtils.copyProperties(vo,careMemberInfo);
        careMemberInfo.setModifyTime(new Date());
        this.careMemberInfoService.updateById(careMemberInfo);

//        List<DeviceVO> hasDeviceVOList = vo.getHasDeviceVOList();
//        //查找原来拥有的设备
//        QueryWrapper<CareDevice> queryWrapper = new QueryWrapper<>();
//        queryWrapper.lambda().eq(CareDevice::getMemberId,vo.getId());
//        List<CareDevice> hasDeviceListOld = this.careDeviceService.list(queryWrapper);
//
//        if (CollUtil.isNotEmpty(hasDeviceListOld)) {
//          for(CareDevice careDevice : hasDeviceListOld){
//              boolean has = false;
//              for(DeviceVO deviceVO : hasDeviceVOList) {
//                  if(careDevice.getId().equals(deviceVO.getId())){
//                      has = true;
//                      break;
//                  }
//              }
//              if(!has) {
//                  //将不再拥有的设备的频安宝用户id置空
//                  this.careDeviceService.updateDevChamb(careDevice.getId());
//              }
//          }
//        }
//
//        //更新拥有的设备
//        if (CollUtil.isNotEmpty(hasDeviceVOList)) {
//            hasDeviceVOList.forEach(item -> {
//                CareDevice careDevice = new CareDevice();
//                BeanUtil.copyProperties(item,careDevice);
//                careDevice.setMemberId(careMemberInfo.getId());
//                careDeviceService.updateById(careDevice);
//            });
//        }

    }
    /**
     * 修改频安宝用户状态
     * @param status
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateCareMemberInfoStatus(Long id, String status){
        //修改频安宝用户
        CareMemberInfo careMemberInfo = this.careMemberInfoService.getById(id);
        careMemberInfo.setStatus(status);
        careMemberInfo.setModifyTime(new Date());
        this.careMemberInfoService.updateById(careMemberInfo);
        return true;
    }

}