package com.care.client.controller;

import com.care.client.service.PassportService;
import com.care.client.service.PinanbaoService;
import com.care.client.vo.HouseContactVO;
import com.care.client.vo.MemberInfoVO;
import com.care.common.annotation.Permission;
import com.care.common.exception.BDException;
import com.care.common.util.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "ContactApiController", tags = { "我的紧急联系人" })
@Slf4j
@RequestMapping("/pinanbao/contact")
public class ContactApiController {

    @Resource
    private PinanbaoService pinanbaoService;
    @Resource
    private PassportService passportService;

    /**
     * 紧急联系人列表
     * @param request
     * @return
     */
    @GetMapping("/my/list")
    @ApiOperation(tags = {"我的紧急联系人"},value = "紧急联系人列表")
    public Result<List<HouseContactVO>> myHouseContactList(HttpServletRequest request, @RequestHeader(value = "token") String token){
        MemberInfoVO current = passportService.checkToken(token);
        try {
            List<HouseContactVO> datas = pinanbaoService.queryContactByMemberId(current.getId());
            return Result.success(datas);
        }catch (BDException e) {
            return  Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("我的紧急联系人: 紧急联系人列表查询出现异常",e);
            return Result.error( "获取失败");
        }
    }


}
