package com.care.client.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.care.client.vo.DeviceMonitorVO;
import com.care.client.vo.HouseContactVO;
import com.care.client.vo.MemberInfoVO;
import com.care.client.vo.OlderVO;
import com.care.common.entity.*;
import com.care.common.enums.OrderStatusEnum;
import com.care.common.enums.OrderTypeEnum;
import com.care.common.enums.RelationTypeEnum;
import com.care.common.exception.BDException;
import com.care.common.service.*;
import com.care.common.vo.device.DeviceVO;
import com.care.common.vo.order.EventOrderVO;
import com.care.common.vo.order.OrderHandleHisVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author: lilt
 * @Date: 2021/6/7
 * @Desc:
 */
@Slf4j
@Service
public class PinanbaoService {
    @Resource
    private CareDeviceService careDeviceService;
    @Resource
    private CareMemberInfoService careMemberInfoService;
    @Resource
    private CareHouseContactService careHouseContactService;
    @Resource
    private CareEventOrderService careEventOrderService;
    @Resource
    private CareEventOrderHandleHisService careEventOrderHandleHisService;
    @Resource
    private FileUploadService fileUploadService;
    @Resource
    private CareHouseService careHouseService;
    @Resource
    private CareOrganizationService careOrganizationService;
    @Resource
    private CareStationService careStationService;
    @Resource
    private CareOlderService careOlderService;
    /**
     * 查询房屋的设备列表
     * @param houseId
     * @return
     */
    public List<DeviceVO> devList(Long houseId){
        return this.careDeviceService.queryListByHouseId(houseId);
    }

    /**
     * 设备监测数据
     * @param devId
     * @return
     */
    public DeviceMonitorVO devMonitorData(Long devId){
        CareDevice device = this.careDeviceService.getById(devId);
        DeviceMonitorVO vo = new DeviceMonitorVO();
        if (device!=null){
            vo.setId(devId);
            vo.setBreathRate(device.getBreathRate());
            vo.setHeartRate(device.getHeartRate());
            vo.setActivityDistance(600);
            vo.setActivityTimeHour(1);
            vo.setActivityTimeMinute(20);
            vo.setSleepTimeHour(8);
            vo.setSleepTimeMinute(7);
        }
        return vo;
    }

    /**
     * 查询房屋紧急联系人
     * @param memberId
     * @return
     */
    public List<HouseContactVO> queryContactByMemberId(Long memberId){
        List<HouseContactVO> vos = Lists.newArrayList();
        CareMemberInfo memberInfo = this.careMemberInfoService.getById(memberId);
        QueryWrapper<CareHouseContact> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareHouseContact::getMemberId, memberInfo.getId())
        .orderByAsc(CareHouseContact::getContactLevel);
        List<CareHouseContact> contacts = this.careHouseContactService.list(queryWrapper);
        if (CollUtil.isNotEmpty(contacts)){
            contacts.forEach(item -> {
                HouseContactVO vo = new HouseContactVO();
                BeanUtil.copyProperties(item,vo);
                vo.setRelationTypeDesc(RelationTypeEnum.getCodeToName(vo.getRelationType()));
                vos.add(vo);
            });
        }
        return vos;
    }

    /**
     * 查询正在进行中的事件
     * @param houseId
     * @return
     */
    public EventOrderVO getDoingEvent(Long houseId){
        EventOrderVO vo = null;
        QueryWrapper<CareEventOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareEventOrder::getHouseId,houseId).and(item->
            item.eq(CareEventOrder::getStatus, OrderStatusEnum.TODO.getValue()).or(i->
                i.eq(CareEventOrder::getStatus,OrderStatusEnum.DOING.getValue())
            )
        ).orderByAsc(CareEventOrder::getCreateTime);
        List<CareEventOrder> list = this.careEventOrderService.list(queryWrapper);
        if (CollUtil.isNotEmpty(list)){
            CareEventOrder order = list.get(0);
            vo = new EventOrderVO();
            BeanUtil.copyProperties(order,vo);
        }
        return vo;
    }

    /**
     * 获取处理记录
     * @return
     */
    public List<OrderHandleHisVO> queryOrderHandleHis(Long orderId){
        List<OrderHandleHisVO> result = Lists.newArrayList();
        QueryWrapper<CareEventOrderHandleHis> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareEventOrderHandleHis::getOrderId,orderId)
                .orderByDesc(CareEventOrderHandleHis::getCreateTime);
        List<CareEventOrderHandleHis> datas = this.careEventOrderHandleHisService.list(queryWrapper);
        if (CollUtil.isNotEmpty(datas)){
            datas.forEach(item ->{
                OrderHandleHisVO hisVO = new OrderHandleHisVO();
                BeanUtil.copyProperties(item,hisVO);
                result.add(hisVO);
            });
        }
        return result;
    }

    /**
     *  老人 SOS呼叫
     * @param member
     * @param file
     * @return
     */
    public boolean sos(MemberInfoVO member,  MultipartFile file){

//        EventOrderVO orderVO = getDoingEvent(member.getHouseId());
//        if (orderVO!=null){
//            throw new BDException("有进行中事件，请勿重复呼叫");
//        }
//
//        Date now  = DateUtil.date();
//        //1. 保存文件
//        String url = fileUploadService.upload(file,"sos");
//        //2. 查询房屋
//        CareHouse house = this.careHouseService.getById(member.getHouseId());
//        //2. 生成工单
//        CareEventOrder order = new CareEventOrder();
//        order.setOrgId(house.getOrgId());
//        order.setOrgName(this.careOrganizationService.getById(house.getOrgId()).getName());
//        order.setStationId(house.getStationId());
//        order.setStationName(this.careStationService.getById(house.getStationId()).getName());
//        order.setHouseId(house.getId());
//        order.setHouseName(house.getName());
//        order.setTitle(house.getAddr()+" "+member.getName()+" 主动呼叫");
//        order.setOlderId(member.getId());
//        order.setOlderName(member.getName());
//        order.setVoiceUrl(url);
//        order.setOrderType(OrderTypeEnum.ZHU_DONG_HU_JIAO.getValue());
//        order.setStatus(OrderStatusEnum.TODO.getValue());
//        order.setCreateTime(now);
//        order.setModifyTime(now);
//        this.careEventOrderService.saveOrder(order);
        return true;
    }

    /**
     *  老人 SOS呼叫
     * @param member
     * @return
     */
    public boolean sos(MemberInfoVO member){

//        EventOrderVO orderVO = getDoingEvent(member.getHouseId());
//        if (orderVO!=null){
//            throw new BDException("有进行中事件，请勿重复呼叫");
//        }
//        Date now  = DateUtil.date();
//        //2. 查询房屋
//        CareHouse house = this.careHouseService.getById(member.getHouseId());
//        //2. 生成工单
//        CareEventOrder order = new CareEventOrder();
//        order.setOrgId(house.getOrgId());
//        order.setOrgName(this.careOrganizationService.getById(house.getOrgId()).getName());
//        order.setStationId(house.getStationId());
//        order.setStationName(this.careStationService.getById(house.getStationId()).getName());
//        order.setHouseId(house.getId());
//        order.setHouseName(house.getName());
//        order.setTitle(house.getAddr()+" "+member.getName()+" 主动呼叫");
//        order.setOlderId(member.getId());
//        order.setOlderName(member.getName());
//        order.setOrderType(OrderTypeEnum.ZHU_DONG_HU_JIAO.getValue());
//        order.setStatus(OrderStatusEnum.TODO.getValue());
//        order.setCreateTime(now);
//        order.setModifyTime(now);
//        this.careEventOrderService.saveOrder(order);
        return true;
    }

    /**
     * 获取3D监控页面URL
     * @return
     */
    public String getRealTimeMonitorUrl(Long devId){
        Map<String,String> map =  this.careDeviceService.get3dUrl(devId);
        return  map.get("toWX");
    }

    /**
     * 查询房屋老人
     * @param memberId
     * @return
     */
    public List<OlderVO> queryOlderByMemberId(Long memberId){
        List<OlderVO> vos = Lists.newArrayList();
        CareMemberInfo memberInfo = this.careMemberInfoService.getById(memberId);
        QueryWrapper<CareOlder> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareOlder::getMemberId, memberInfo.getId());
        List<CareOlder> contacts = this.careOlderService.list(queryWrapper);
        if (CollUtil.isNotEmpty(contacts)){
            contacts.forEach(item -> {
                OlderVO vo = new OlderVO();
                BeanUtil.copyProperties(item,vo);
                vos.add(vo);
            });
        }
        return vos;
    }
}
