package com.care.common.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.URLUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.care.common.entity.CareDevice;
import com.care.common.enums.ChambOrderStatusEnum;
import com.care.common.enums.DeviceActiveStatusEnum;
import com.care.common.enums.DeviceStatusEnum;
import com.care.common.mapper.CareDeviceMapper;
import com.care.common.service.CareDeviceService;
import com.care.common.vo.PageReqVO;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.device.DeviceVO;
import com.care.common.vo.order.ChambEventOrderVO;
import com.google.common.collect.Maps;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Map;

/**
 * 被监护人-设备(CareDevice)表服务实现类
 *
 * @author makejava
 * @since 2021-05-21 00:08:37
 */
@Service
public class CareDeviceServiceImpl extends ServiceImpl<CareDeviceMapper, CareDevice> implements CareDeviceService {
    @Value("${device.3d.url.host:#{null}}")
    private String url3DUrlHost;
    @Value("${device.3d.url.path:#{null}}")
    private String url3DUrlPath;
    @Override
    public Map<String,String> get3dUrl(Long deviceId) {
        Map<String,String> map = Maps.newHashMap();
        CareDevice device = this.baseMapper.selectById(deviceId);
        if (device!=null && StrUtil.isNotEmpty(device.getTopic())){
            try {
                String topic = java.net.URLEncoder.encode(device.getDevCode(),"UTF-8");
                String toWx =   url3DUrlHost + url3DUrlPath + topic;
                map.put("toWX",toWx);
                map.put("toBms", url3DUrlPath + topic);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
        }
        return map;
    }
    /**
     * 查询房屋的设备列表
     *
     * @param houseId
     * @return
     */
    @Override
    public List<DeviceVO> queryListByHouseId(Long houseId) {
        return this.baseMapper.queryListByHouseId(houseId);
    }

    @Override
    public IPage<DeviceVO> queryList(PageReqVO pageReqVo, Long stationId, String devCode, String activeStatus,String chambName, String status, String houseAddr,Long chambId, Long memberId,UserLogindConvertVO loginUser) {
        IPage<DeviceVO> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());
        IPage<DeviceVO> pageRes = this.baseMapper.queryList(page, stationId,devCode,activeStatus,chambName,status,houseAddr,chambId,memberId,loginUser.getOrgId(),loginUser.getStationId());
        if(CollUtil.isNotEmpty(pageRes.getRecords())){
            pageRes.getRecords().forEach(item -> {
                item.setStatusName(DeviceStatusEnum.getCodeToName(item.getStatus()));
                item.setActiveStatusName(DeviceActiveStatusEnum.getCodeToName(item.getActiveStatus()));
            });
        }
        return pageRes;
    }

    @Override
    public void updateDevChamb(Long id) {
        this.baseMapper.updateDevChamb(id);
    }

    public static void main(String[] args) throws UnsupportedEncodingException {
        System.out.println(java.net.URLEncoder.encode("5JPD/monitor/f1/event","UTF-8"));
    }
}