package com.care.keeper.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.care.common.cache.RedisKeyConstant;
import com.care.common.cache.RedisUtil;
import com.care.common.entity.CareSysUser;
import com.care.common.service.CareSysUserService;
import com.care.common.util.Result;
import com.care.keeper.vo.KeeperInfoVO;
import com.care.keeper.vo.KeeperInitParams;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Map;

@Slf4j
@Service
public class AbstractKeeperPassportService {

    @Resource
    private RedisUtil redisUtil;


    @Resource
    private CareSysUserService careSysUserService;

    public Result<Object> cache(KeeperInitParams params) {
        String token = IdUtil.fastUUID();
        // cache member
        CareSysUser member = null;

        if ("wx".equalsIgnoreCase(params.getLoginType())) {
            member = careSysUserService.detailByOpenid4Keeper(params.getOpenid());
            if (member == null) {
                member = careSysUserService.detailByPhone4Keeper(params.getPhone());
                if (member == null) {
                    return Result.error("账号不存在！");
                }
            }
        } else if ("pwd".equalsIgnoreCase(params.getLoginType())) {
            member = careSysUserService.detailByPhone4Keeper(params.getPhone());
            if (member == null) {
                return Result.error("账号密码不匹配");
            }
            if (!StrUtil.equalsIgnoreCase(SecureUtil.md5(params.getPassword()), member.getPassword())) {
                return Result.error("账号密码不匹配");
            }
        }
        if (StringUtils.isBlank(member.getOpenId()) && StringUtils.isNotBlank(params.getOpenid())) {
            UpdateWrapper<CareSysUser> careMemberInfoUpdateWrapper = new UpdateWrapper<>();
            careMemberInfoUpdateWrapper.lambda().eq(CareSysUser::getId,member.getId()).set(CareSysUser::getOpenId,params.getOpenid());
            boolean bindFlag = this.careSysUserService.update(careMemberInfoUpdateWrapper);
            if (bindFlag) {
                member.setOpenId(params.getOpenid());
            }
        }
        KeeperInfoVO keeperInfoVO = new KeeperInfoVO();
        BeanUtil.copyProperties(member,keeperInfoVO);
        redisUtil.hset(RedisKeyConstant.PINANSHOUHU_LOGIN_INFO, token, keeperInfoVO, RedisKeyConstant.PINANSHOUHU_LOGIN_INFO_TIME);

        Map<String,Object> data = Maps.newHashMap();
        data.put("token", token);
        data.put("member", keeperInfoVO);
        return Result.success(data);
    }

    public KeeperInfoVO refreshMember(String token) {
        if (StrUtil.isNotBlank(token) && redisUtil.hHasKey(RedisKeyConstant.PINANSHOUHU_LOGIN_INFO, token)) {
            KeeperInfoVO oldMemberInfoVo = (KeeperInfoVO)redisUtil.hget(RedisKeyConstant.PINANSHOUHU_LOGIN_INFO, token);
            CareSysUser member = careSysUserService.getById(oldMemberInfoVo.getId());
            if (oldMemberInfoVo != null && member != null) {
                KeeperInfoVO memberInfoVO = new KeeperInfoVO();
                BeanUtil.copyProperties(member,memberInfoVO);
                redisUtil.hset(RedisKeyConstant.PINANSHOUHU_LOGIN_INFO, token, memberInfoVO, RedisKeyConstant.PINANSHOUHU_LOGIN_INFO_TIME);
                return memberInfoVO;
            }
        }
        return null;
    }
}
